@@ -13,7 +13,10 @@ use crate::{
     Result,
 };
 use sn_messaging::{
-    node::{JoinResponse, Proposal, RelocatePromise, RoutingMsg, SignedShare, Variant},
+    node::{
+        JoinAsRelocatedResponse, JoinResponse, Proposal, RelocatePromise, RoutingMsg, SignedShare,
+        Variant,
+    },
     DstLocation,
 };
 use xor_name::XorName;
@@ -47,6 +50,11 @@ impl Core {
                     return Ok(MessageStatus::Useless);
                 }
             }
+            Variant::JoinAsRelocatedRequest(req) => {
+                if !self.is_elder() && req.section_key == *self.section.chain().last_key() {
+                    return Ok(MessageStatus::Useless);
+                }
+            }
             Variant::DkgStart {
                 elder_candidates, ..
             } => {
@@ -64,6 +72,15 @@ impl Core {
                 }
                 JoinResponse::ResourceChallenge { .. } => {}
             },
+            Variant::JoinAsRelocatedResponse(resp) => match **resp {
+                JoinAsRelocatedResponse::Approval { .. }
+                | JoinAsRelocatedResponse::Retry(_)
+                | JoinAsRelocatedResponse::Redirect(_)
+                | JoinAsRelocatedResponse::NodeNotReachable(_) => {
+                    // Skip validation of these. We will validate them inside the relocation task.
+                    return Ok(MessageStatus::Useful);
+                }
+            },
             Variant::Sync { section, .. } => {
                 // Ignore `Sync` not for our section.
                 if !section.prefix().matches(&self.node.name()) {
